/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.hdiv.taglib.util;

import java.util.Enumeration;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;

import org.apache.struts.util.RequestUtils;
import org.hdiv.dataComposer.IDataComposer;
import org.hdiv.taglib.util.RequestUtilsHDIV;
import org.hdiv.util.HDIVUtil;
import org.hdiv.config.HDIVConfig;

/**
 * General purpose utility methods related to processing a servlet request.
 * 
 * @author Gorka Vicente
 * @see org.apache.struts.util.RequestUtils
 */
public class RequestUtilsHDIV extends RequestUtils {

	/**
	 * Adds HDIV state as a parameter if <code>url</code> references our
	 * application.
	 * 
	 * @param request HTTP request
	 * @param response The servlet response we are creating
	 * @param url URL
	 * @return <code>url</code> with the HDIV state added as a new parameter
	 * @since HDIV 2.0.1
	 */
	public static String addHDIVParameterIfNecessary(HttpServletRequest request, HttpServletResponse response,
			String url) {

		String hdivUrl = url;
		try {
			String urlWithoutRelativePaths = RequestUtilsHDIV.getURLWithoutRelativePaths(request, hdivUrl);
			if (urlWithoutRelativePaths != null) {

				String encodedURL = RequestUtilsHDIV.encodeValues(hdivUrl, urlWithoutRelativePaths, response
						.getCharacterEncoding());
				hdivUrl = RequestUtilsHDIV.addHDIVState(encodedURL, null);
			}
		} catch (JspException e) {
			return url;
		}
		return hdivUrl;
	}

	/**
	 * Generate url with encoded parameters.
	 * 
	 * @param url URL to encode
	 * @param urlWithoutRelativePaths URL without relative paths
	 * @param charEncoding character encoding
	 * @return URL with encoded parameters
	 * @throws JspException if a JSP exception has occurred
	 */
	public static String encodeValues(String url, String urlWithoutRelativePaths, String charEncoding)
			throws JspException {

		String encodedURL = url;
		IDataComposer dataComposer = HDIVUtil.getDataComposer();

		String actionMappingName = null;
		if ((urlWithoutRelativePaths == null) || (url.equals(urlWithoutRelativePaths))) {
			actionMappingName = HDIVUtil.getActionMappingName(url);
		} else {
			actionMappingName = HDIVUtil.getActionMappingName(urlWithoutRelativePaths);
		}

		dataComposer.beginRequest(actionMappingName);

		int question = encodedURL.indexOf("?");
		if (question > 0) { // link has parameters to encode
			encodedURL = HDIVUtil.composeAction(encodedURL, question, charEncoding);
		}
		return encodedURL;
	}

	/**
	 * It deletes from the URL <code>url</code> all the references to relative
	 * directories.
	 * 
	 * @param HttpServletRequest request HTTP request
	 * @param url URL
	 * @return If the <code>url</code> has references to relative directories,
	 *         it returns the absolute path deleting these relative references.
	 *         If the <code>url</code> is relative and it does not reference
	 *         our application, it will return null. If the <code>url</code>
	 *         is not relative, it is returned as it is.
	 * @throws JspException if a JSP exception has occurred
	 */
	public static String getURLWithoutRelativePaths(HttpServletRequest request, String url) throws JspException {

		if (!hasActionOrServletExtension(url)) {
			return null;
		}

		String contextName = request.getContextPath();
		if (contextName.startsWith("/"))
			contextName = contextName.substring(1);

		int contextStart = url.indexOf(contextName);

		// If it is an action and it is directed to our application we handle
		// it.
		if (contextStart >= 0) {
			return url;

		} else {
			// we check if it is a relative url. If so, as the name of the
			// application context is not visible in the url, we must check if
			// this
			// url references our domain.
			boolean absolute = url.startsWith("/");
			if (!absolute) {
				String relativePath = RequestUtilsHDIV.getRelativePath(request, url, contextName);
				return (relativePath != null) ? relativePath : null;
			}

			return null;
		}
	}

	/**
	 * Checks if <code>path</code> has an action extension or a jsp page
	 * extension.
	 * 
	 * @param path path
	 * @return True if <code>path</code> is an action or references a jsp
	 *         page.
	 * @since HDIV 2.0.4
	 */
	public static boolean hasActionOrServletExtension(String path) {

		if (path.indexOf("?") > 0) {
			path = path.substring(0, path.indexOf("?"));
		}

		if (path.charAt(path.length() - 1) == '/') {
			return true;
		}

		int pound = path.indexOf("#");
		if (pound >= 0) {
			path = path.substring(0, pound);
		}

		// strip a servlet session ID from
		path = HDIVUtil.stripSession(path);

		if (path.endsWith(".jsp")) {
			return true;
		}

		HDIVConfig hdivConfig = (HDIVConfig) HDIVUtil.getApplication().getBean("config");
		if (hdivConfig.getProtectedURLPatterns() != null) {

			for (Enumeration extensionsIds = hdivConfig.getProtectedURLPatterns().elements(); extensionsIds
					.hasMoreElements();) {

				Pattern extensionPattern = (Pattern) extensionsIds.nextElement();
				Matcher m = extensionPattern.matcher(path);

				if (m.matches()) {
					return true;
				}
			}
		}

		return (!path.startsWith("/")) && (path.indexOf(".") == -1);
	}

	/**
	 * It deletes from the URL <code>url</code> all the references to relative
	 * directories.
	 * 
	 * @param HttpServletRequest request HTTP request
	 * @param url URL
	 * @param contextName application context
	 * @return If the <code>url</code> has references to relative directories,
	 *         it returns the absolute path deleting these relative references.
	 *         If the <code>url</code> is relative and it does not reference
	 *         our application, it will return null.
	 * @throws JspException if a JSP exception has occurred
	 */
	public static String getRelativePath(HttpServletRequest request, String url, String contextName)
			throws JspException {

		try {
			String servletAbsolutePath = request.getContextPath() + request.getServletPath();
			String currentServletPath = servletAbsolutePath.substring(0, servletAbsolutePath.lastIndexOf("/"));

			String currentURL = url;
			while (currentURL.startsWith("..")) {
				currentServletPath = currentServletPath.substring(0, currentServletPath.lastIndexOf("/"));
				currentURL = currentURL.substring(3);
			}

			if (currentServletPath.indexOf(contextName) >= 0) {
				return currentServletPath + "/" + currentURL;
			}

		} catch (Exception e) {
			throw new JspException(e.getMessage());
		}

		return null;
	}

	/**
	 * Adds the HDIV parameter, depending on the strategu defined by the user,
	 * to validate the request <code>encodedURL</code>.
	 * 
	 * @param encodedURL URL encoded
	 * @see org.hdiv.composer.IDataComposer
	 */
	public static String addHDIVState(String encodedURL, String anchor) {

		String hdivParameter = (String) HDIVUtil.getHttpSession().getAttribute("HDIVParameter");

		String separator = "";
		IDataComposer dataComposer = HDIVUtil.getDataComposer();
		String requestId = dataComposer.endRequest();

		if ((requestId.length() <= 0) || (encodedURL.startsWith("javascript:"))) {
			return encodedURL;
		}

		// we ask if the link has an anchor. If so, we must remove it to be
		// added
		// later on, once the hdivParameter has been added. this way we get a
		// link
		// like this: ../action.do?parameters&hdivParameter=2-5#anchor
		boolean isAnchor = (anchor != null) && (!anchor.equals(""));
		if (isAnchor) {
			encodedURL = encodedURL.replaceFirst("#" + anchor, "");
		}

		// we check if the url contains parameters
		separator = (encodedURL.indexOf("?") > 0) ? "&" : "?";

		hdivParameter = separator + hdivParameter + "=" + requestId;

		return encodedURL + hdivParameter + ((isAnchor) ? "#" + anchor : "");
	}

	/**
	 * It generates a new encoded values for the <code>url</code> parameters.
	 * 
	 * @param HttpServletRequest request HTTP request
	 * @param url request url
	 * @param charEncoding character encoding
	 * @return URL with encoded parameters
	 * @throws JspException if a JSP exception has occurred
	 */
	public static String encodeURLParameters(HttpServletRequest request, String url, String charEncoding)
			throws JspException {

		String urlWithoutRelativePaths = RequestUtilsHDIV.getURLWithoutRelativePaths(request, url);

		if (urlWithoutRelativePaths == null) {
			return url;
		}

		return RequestUtilsHDIV.encodeValues(url, urlWithoutRelativePaths, charEncoding);
	}

}
